import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { useCustomRender } from '../hooks/config';
import { unreachable, asTitle } from '../tools';
import { JsonComp } from './Json';
import { LinkRender } from './link';
export var DisplayMode;
(function (DisplayMode) {
    DisplayMode["auto"] = "auto";
    DisplayMode["plain"] = "plain";
    DisplayMode["datetime"] = "datetime";
    DisplayMode["date"] = "date";
    DisplayMode["duration"] = "duration";
    DisplayMode["as_title"] = "as_title";
    DisplayMode["markdown"] = "markdown";
    DisplayMode["json"] = "json";
    DisplayMode["inline_code"] = "inline_code";
})(DisplayMode || (DisplayMode = {}));
export const DisplayComp = (props) => {
    const CustomRenderComp = useCustomRender(props);
    if (CustomRenderComp) {
        return _jsx(CustomRenderComp, {});
    }
    const { onClick } = props;
    if (onClick) {
        return (_jsx(LinkRender, { onClick: onClick, children: _jsx(DisplayRender, { ...props }) }));
    }
    else {
        return _jsx(DisplayRender, { ...props });
    }
};
const DisplayRender = (props) => {
    const mode = props.mode ?? DisplayMode.auto;
    const value = props.value ?? null;
    if (mode === DisplayMode.json) {
        return _jsx(JsonComp, { type: "JSON", value: value });
    }
    else if (Array.isArray(value)) {
        return _jsx(DisplayArray, { type: "DisplayArray", mode: mode, value: value });
    }
    else if (typeof value === 'object' && value !== null) {
        return _jsx(DisplayObject, { type: "DisplayObject", mode: mode, value: value });
    }
    else {
        return _jsx(DisplayPrimitive, { type: "DisplayPrimitive", mode: mode, value: value });
    }
};
export const DisplayArray = (props) => {
    const CustomRenderComp = useCustomRender(props);
    if (CustomRenderComp) {
        return _jsx(CustomRenderComp, {});
    }
    const { mode, value } = props;
    return (_jsx(_Fragment, { children: value.map((v, i) => (_jsxs("span", { children: [_jsx(DisplayComp, { type: "Display", mode: mode, value: v }), ",", ' '] }, i))) }));
};
export const DisplayObject = (props) => {
    const CustomRenderComp = useCustomRender(props);
    if (CustomRenderComp) {
        return _jsx(CustomRenderComp, {});
    }
    const { mode, value } = props;
    return (_jsx(_Fragment, { children: Object.entries(value).map(([key, v], i) => (_jsxs("span", { children: [key, ": ", _jsx(DisplayComp, { type: "Display", mode: mode, value: v }, i), ",", ' '] }, key))) }));
};
export const DisplayPrimitive = (props) => {
    const CustomRenderComp = useCustomRender(props);
    if (CustomRenderComp) {
        return _jsx(CustomRenderComp, {});
    }
    const { mode, value } = props;
    switch (mode) {
        case DisplayMode.auto:
            return _jsx(DisplayAuto, { value: value });
        case DisplayMode.plain:
            return _jsx(DisplayPlain, { value: value });
        case DisplayMode.datetime:
            return _jsx(DisplayDateTime, { value: value });
        case DisplayMode.date:
            return _jsx(DisplayDate, { value: value });
        case DisplayMode.duration:
            return _jsx(DisplayDuration, { value: value });
        case DisplayMode.as_title:
            return _jsx(DisplayAsTitle, { value: value });
        case DisplayMode.markdown:
            return _jsx(DisplayMarkdown, { value: value });
        case DisplayMode.json:
        case DisplayMode.inline_code:
            return _jsx(DisplayInlineCode, { value: value });
        default:
            unreachable('Unexpected display type', mode, props);
    }
};
const DisplayNull = () => {
    return _jsx("span", { className: "fu-null", children: "\u2014" });
};
const DisplayAuto = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else if (typeof value === 'boolean') {
        if (value) {
            return _jsx(_Fragment, { children: "\u2713" });
        }
        else {
            return _jsx(_Fragment, { children: "\u00D7" });
        }
    }
    else if (typeof value === 'number') {
        return _jsx(_Fragment, { children: value.toLocaleString() });
    }
    else {
        return _jsx(_Fragment, { children: value });
    }
};
const DisplayPlain = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        return _jsx(_Fragment, { children: value.toString() });
    }
};
const DisplayDateTime = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        // TODO
        return _jsx(_Fragment, { children: value.toString() });
    }
};
const DisplayDate = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        // TODO
        return _jsx(_Fragment, { children: value.toString() });
    }
};
const DisplayDuration = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        // TODO
        return _jsx(_Fragment, { children: value.toString() });
    }
};
const DisplayAsTitle = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        return _jsx(_Fragment, { children: asTitle(value.toString()) });
    }
};
const DisplayMarkdown = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        // TODO
        return _jsx(_Fragment, { children: value.toString() });
    }
};
const DisplayInlineCode = ({ value }) => {
    if (value === null) {
        return _jsx(DisplayNull, {});
    }
    else {
        return _jsx("code", { className: "fu-inline-code", children: value.toString() });
    }
};
export function renderEvent(event, data) {
    let newEvent = event ? { ...event } : undefined;
    if (newEvent) {
        if (newEvent.type === 'go-to' && newEvent.url) {
            // for go-to events with a URL, substitute the row values into the url
            const url = subKeys(newEvent.url, data);
            if (url === null) {
                newEvent = undefined;
            }
            else {
                newEvent.url = url;
            }
        }
    }
    return newEvent;
}
const subKeys = (template, row) => {
    let returnNull = false;
    const r = template.replace(/{(.+?)}/g, (_, key) => {
        const v = row[key];
        if (v === undefined) {
            throw new Error(`field "${key}" not found in ${JSON.stringify(row)}`);
        }
        else if (v === null) {
            returnNull = true;
            return 'null';
        }
        else {
            return v.toString();
        }
    });
    if (returnNull) {
        return null;
    }
    else {
        return r;
    }
};
