import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { useContext } from 'react';
import { ErrorContext } from '../hooks/error';
import { useCustomRender } from '../hooks/config';
import { unreachable } from '../tools';
import { TextComp } from './text';
import { ParagraphComp } from './paragraph';
import { PageTitleComp } from './PageTitle';
import { DivComp } from './div';
import { HeadingComp } from './heading';
import { MarkdownComp } from './Markdown';
import { CodeComp } from './Code';
import { FormComp } from './form';
import { FormFieldInputComp, FormFieldBooleanComp, FormFieldSelectComp, FormFieldSelectSearchComp, FormFieldFileComp, } from './FormField';
import { ButtonComp } from './button';
import { LinkComp, LinkRender } from './link';
import { LinkListComp } from './LinkList';
import { NavbarComp } from './navbar';
import { ModalComp } from './modal';
import { TableComp } from './table';
import { PaginationComp } from './pagination';
import { DetailsComp } from './details';
import { DisplayArray, DisplayComp, DisplayObject, DisplayPrimitive, } from './display';
import { JsonComp } from './Json';
import { ServerLoadComp } from './ServerLoad';
import { ImageComp } from './image';
import { IframeComp } from './Iframe';
import { CustomComp } from './Custom';
// TODO some better way to export components
export { LinkComp, LinkRender };
export const AnyCompList = ({ propsList }) => (_jsx(_Fragment, { children: propsList.map((child, i) => (_jsx(AnyComp, { ...child }, i))) }));
export const AnyComp = (props) => {
    const { DisplayError } = useContext(ErrorContext);
    const CustomRenderComp = useCustomRender(props);
    if (CustomRenderComp) {
        return _jsx(CustomRenderComp, {});
    }
    const { type } = props;
    try {
        switch (type) {
            case 'Text':
                return _jsx(TextComp, { ...props });
            case 'Paragraph':
                return _jsx(ParagraphComp, { ...props });
            case 'PageTitle':
                return _jsx(PageTitleComp, { ...props });
            case 'Div':
            case 'Page':
                return _jsx(DivComp, { ...props });
            case 'Heading':
                return _jsx(HeadingComp, { ...props });
            case 'Markdown':
                return _jsx(MarkdownComp, { ...props });
            case 'Code':
                return _jsx(CodeComp, { ...props });
            case 'Button':
                return _jsx(ButtonComp, { ...props });
            case 'Link':
                return _jsx(LinkComp, { ...props });
            case 'LinkList':
                return _jsx(LinkListComp, { ...props });
            case 'Navbar':
                return _jsx(NavbarComp, { ...props });
            case 'Form':
            case 'ModelForm':
                return _jsx(FormComp, { ...props });
            case 'FormFieldInput':
                return _jsx(FormFieldInputComp, { ...props });
            case 'FormFieldBoolean':
                return _jsx(FormFieldBooleanComp, { ...props });
            case 'FormFieldFile':
                return _jsx(FormFieldFileComp, { ...props });
            case 'FormFieldSelect':
                return _jsx(FormFieldSelectComp, { ...props });
            case 'FormFieldSelectSearch':
                return _jsx(FormFieldSelectSearchComp, { ...props });
            case 'Modal':
                return _jsx(ModalComp, { ...props });
            case 'Table':
                return _jsx(TableComp, { ...props });
            case 'Pagination':
                return _jsx(PaginationComp, { ...props });
            case 'Details':
                return _jsx(DetailsComp, { ...props });
            case 'Display':
                return _jsx(DisplayComp, { ...props });
            case 'DisplayArray':
                return _jsx(DisplayArray, { ...props });
            case 'DisplayObject':
                return _jsx(DisplayObject, { ...props });
            case 'DisplayPrimitive':
                return _jsx(DisplayPrimitive, { ...props });
            case 'JSON':
                return _jsx(JsonComp, { ...props });
            case 'ServerLoad':
                return _jsx(ServerLoadComp, { ...props });
            case 'Image':
                return _jsx(ImageComp, { ...props });
            case 'Iframe':
                return _jsx(IframeComp, { ...props });
            case 'Custom':
                return _jsx(CustomComp, { ...props });
            default:
                unreachable('Unexpected component type', type, props);
                return _jsx(DisplayError, { title: "Invalid Server Response", description: `Unknown component type: "${type}"` });
        }
    }
    catch (e) {
        // TODO maybe we shouldn't catch this error (by default)?
        const description = e.message;
        return _jsx(DisplayError, { title: "Render Error", description: description });
    }
};
