import { useContext, useState, useEffect, useCallback } from 'react';
import { LocationContext } from './hooks/locationContext';
function pageEventType(event) {
    return `fastui:${event.name}`;
}
export function useFireEvent() {
    const location = useContext(LocationContext);
    function fireEvent(event) {
        if (!event) {
            return;
        }
        console.debug('firing event', event);
        const { type } = event;
        switch (type) {
            case 'page': {
                if (event.pushPath) {
                    location.gotoCosmetic(event.pushPath);
                }
                const detail = { clear: event.clear || false, context: event.context };
                document.dispatchEvent(new CustomEvent(pageEventType(event), { detail }));
                break;
            }
            case 'go-to':
                if (event.url) {
                    location.goto(event.url);
                }
                if (event.query) {
                    location.setQuery(event.query);
                }
                break;
            case 'back':
                location.back();
                break;
        }
    }
    return { fireEvent };
}
export const loadEvent = 'fastui:load';
export function fireLoadEvent(detail) {
    document.dispatchEvent(new CustomEvent(loadEvent, { detail }));
}
export function usePageEventListen(event, initialContext = null) {
    const [eventContext, setEventContext] = useState(initialContext);
    const onEvent = useCallback((e) => {
        const { context, clear } = e.detail;
        if (clear) {
            setEventContext(null);
        }
        else {
            setEventContext(context ?? {});
        }
    }, []);
    useEffect(() => {
        if (!event) {
            return;
        }
        const eventType = pageEventType(event);
        document.addEventListener(eventType, onEvent);
        return () => document.removeEventListener(eventType, onEvent);
    }, [event, onEvent]);
    return {
        eventContext,
        clear: useCallback(() => setEventContext(null), []),
    };
}
